@@ -6,11 +6,11 @@ use warnings;
 require Exporter;
 
 use vars qw($VERSION);
-$VERSION = 2015.070902;
+$VERSION = 2015.071401;
 
 require Exporter;
 our @ISA = qw(Exporter);
-our @EXPORT_OK = qw(TiedRank EstimateTiedRank);
+our @EXPORT_OK = qw(TiedRank EstimateTiedRank EstimateTiedRankWithDups) ;
 
 use PDL;
 
@@ -58,6 +58,52 @@ sub EstimateTiedRank{#Estimate the tied rank of an unseen instance
     return ($inst_ranked,$inst_uniqueness);    
 }
 
+sub EstimateTiedRankWithDups{#Estimate the tied rank of an unseen instance    
+    my $insts =  $_[0]; #Unseen instances
+    my $mypdl =  $_[1]; #Original Dataset of which Tied Ranks are already calculated
+    my $ranked_pdl =  $_[2]; # Tied Ranked dataset of $mypdl
+    my $duplicates_pdl =  $_[3]; # Number of duplicates in the tied Ranked dataset of $mypdl
+    
+    my $rows = $mypdl->getdim(0);
+    my $cols = $mypdl->getdim(1);
+    
+    my $insts_rows = $insts->getdim(0);
+    
+    my $inst_ranked = PDL->zeros($insts_rows,$cols) -1;
+    my $inst_duplicates = PDL->ones($insts_rows,$cols);
+    
+    
+    for (my $this_col=0; $this_col < $cols; $this_col++){
+	    my $original_values = $mypdl->dice_axis(1,$this_col)->flat;
+	    my $sorted_indx = qsorti($original_values);
+	    my $sorted_values = $original_values->dice($sorted_indx);
+	    
+        for (my $this_inst_raw=0; $this_inst_raw < $insts_rows; $this_inst_raw++){		
+	    my $this_value = $insts->dice($this_inst_raw,$this_col)->flat;	    
+	    
+	    for (my $i = 0; $i < $rows; $i++){
+		my $sorted_val = $sorted_values->dice($i)->flat;
+		if ($this_value == $sorted_val) {
+		    SetValue($inst_ranked->dice($this_inst_raw,$this_col),
+                             $ranked_pdl->dice($sorted_indx->dice($i),$this_col));
+            SetValue($inst_duplicates->dice($this_inst_raw,$this_col),
+                            $duplicates_pdl->dice($sorted_indx->dice($i),$this_col));                            
+		    last;
+		}elsif ($this_value < $sorted_val) {		    
+		    SetValue($inst_ranked->dice($this_inst_raw,$this_col),$i);
+		    last;
+		}		
+	    }
+	    if ($inst_ranked->dice($this_inst_raw,$this_col) == -1) {
+		SetValue($inst_ranked->dice($this_inst_raw,$this_col),$rows);
+	    }
+	    	    
+	}
+    }
+    
+    return ($inst_ranked,$inst_duplicates);    
+}
+
 sub TiedRank{    #Calculate Tied Ranks
     my $mypdl =  $_[0];
     my $rows = $mypdl->getdim(0);
@@ -171,6 +217,13 @@ EstimateTiedRank returns two PDL varibles each of the same size as the test data
 The first varible contains the lowest nearest ranks from the tied ranks of the training data.
 The second variable contains whether the value is unique, ie. to be unique it should not exist in the training dataset in the corresponding attribute.
 
+EstimateTiedRankWithDups
+
+This produce similar functionality to the EstimateTiedRank.
+However additionally it needs number of duplicates as returned by TiedRank.
+It retunes the duplicate count in the traning data instead of the uniqueness.
+Therefore, if a value in the second retuned parameter (duplicate count) is zero, the corresponding value in the test/evaluation data is unique.
+
 DEPENDENCIES
 
 This module requires these other modules and libraries:
